#pragma once


#include "FreeRTOS.h"         //#include <arpa/inet.h>
#include "FreeRTOS_IP.h"      //#include <netinet/in.h>
#include "FreeRTOS_sockets.h" //#include <sys/socket.h>
#include "common_utils.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#if defined(__ICCARM__) || defined( __CC_ARM ) || defined( __ARMCC_VERSION )
typedef	unsigned short	u_short;
typedef signed long     ssize_t;
#define getRandom() rand()
#else
#include <unistd.h>
#define getRandom() random()
#endif

#define UDPSOCKET SOCKET

typedef Socket_t SOCKET;
//typedef Socket_t UDPSOCKET;
typedef uint32_t IPADDRESS; // On linux use uint32_t in network byte order (per getpeername)
typedef uint16_t IPPORT; // on linux use network byte order

#define NULLSOCKET 0

inline void closesocket(SOCKET s) {
    //FreeRTOS_shutdown(s);
    //while( FreeRTOS_recv( xSocket, pcBufferToTransmit, xTotalLengthToSend, 0 ) >= 0 )
    FreeRTOS_closesocket(s);
}

inline void socketpeeraddr(SOCKET s, IPADDRESS *addr, IPPORT *port) {

    struct freertos_sockaddr r;
    if(FreeRTOS_GetRemoteAddress(s, &r) < 0) {
        APP_PRINT("getpeername failed\n");
        *addr = 0;
        *port = 0;
    }
    else {
        //htons

        *port  = r.sin_port;
        *addr = r.sin_addr;
    }
}

inline void udpsocketclose(UDPSOCKET s) {
    //FreeRTOS_shutdown(s);
    //
    FreeRTOS_closesocket(s);
}

inline UDPSOCKET udpsocketcreate(unsigned short portNum)
{
    struct freertos_sockaddr addr;

    addr.sin_family      = FREERTOS_AF_INET;

    Socket_t s     = FreeRTOS_socket(FREERTOS_AF_INET, FREERTOS_SOCK_DGRAM, FREERTOS_IPPROTO_UDP );
    addr.sin_port = FreeRTOS_htons(portNum);
    if (FreeRTOS_bind(s, &addr, sizeof(addr)) != 0) {
        //APP_PRINT("Error, can't bind\n")
        FreeRTOS_closesocket(s);
        s = NULL;
    }

    return s;
}

// TCP sending
inline ssize_t socketsend(SOCKET sockfd, const void *buf, size_t len)
{
    // printf("TCP send\n");
    return FreeRTOS_send(sockfd, buf, len, 0);
}

inline ssize_t udpsocketsend(UDPSOCKET sockfd, const void *buf, size_t len,
                             IPADDRESS destaddr, uint16_t destport)
{
    struct freertos_sockaddr addr;

    addr.sin_family = FREERTOS_AF_INET;
    addr.sin_addr   = destaddr;
    addr.sin_port   = FreeRTOS_htons(destport);
    //printf("UDP send to 0x%0x:%0x\n", destaddr, destport);

    return FreeRTOS_sendto(sockfd, buf, len, 0, &addr, sizeof(addr));
}

/**
   Read from a socket with a timeout.

   Return 0=socket was closed by client, -1=timeout, >0 number of bytes read
 */
inline int socketread(SOCKET sock, char *buf, size_t buflen, int timeoutmsec)
{
    if(sock == NULL)
        return 0;

    TickType_t tv = pdMS_TO_TICKS(timeoutmsec);
    // Use a timeout on our socket read to instead serve frames
    FreeRTOS_setsockopt(sock, 0, FREERTOS_SO_RCVTIMEO, &tv, 0);

    BaseType_t res = FreeRTOS_recv(sock,buf,buflen,0);
    if(res > 0) {
        return res;
    }
    else if(res == 0) {
        return -pdFREERTOS_ERRNO_EAGAIN; // no data
    }
    else {
        if (res == -pdFREERTOS_ERRNO_EWOULDBLOCK || res == -pdFREERTOS_ERRNO_EAGAIN)
        {
            return -1;
        }
        else
        {
            return res; // unknown error, just claim client dropped it
        }
    }
}
