#include "common_utils.h"

#define NUM_STRING_DESCRIPTOR               (7U)

/* Global variables */
extern uint8_t g_apl_device[];
extern uint8_t g_apl_configuration[];
extern uint8_t g_apl_hs_configuration[];
extern uint8_t g_apl_qualifier_descriptor[];
extern uint8_t *g_apl_string_table[];

const usb_descriptor_t g_usb_descriptor =
{
g_apl_device,                   /* Pointer to the device descriptor */
g_apl_configuration,            /* Pointer to the configuration descriptor for Full-speed */
g_apl_hs_configuration,         /* Pointer to the configuration descriptor for Hi-speed */
g_apl_qualifier_descriptor,     /* Pointer to the qualifier descriptor */
g_apl_string_table,             /* Pointer to the string descriptor table */
NUM_STRING_DESCRIPTOR
};

FSP_CPP_HEADER
void R_BSP_WarmStart(bsp_warm_start_event_t event);
FSP_CPP_FOOTER

/*******************************************************************************************************************//**
 * main() is generated by the RA Configuration editor and is used to generate threads if an RTOS is used.  This function
 * is called by main() when no RTOS is used.
 **********************************************************************************************************************/
void hal_entry(void)
{
    bool card = false;
    sdmmc_status_t Sts;
    usb_status_t event;
    fsp_pack_version_t version = {RESET_VALUE};
    volatile fsp_err_t err = FSP_SUCCESS;

    /* version get API for FLEX pack information */
    R_FSP_VersionGet(&version);

    /* Example Project information printed on the Console */
    APP_PRINT(BANNER_INFO, EP_VERSION, version.major, version.minor, version.patch);

    /* Open SD card & check if it's in the slot*/
    err = R_SDHI_Open(&g_sdmmc_ctrl, &g_sdmmc_cfg);
    if (FSP_SUCCESS != err)
    {
        APP_ERR_PRINT ("\r\nError in initializing SDHI\r\n")
    }
    else
    {
        APP_PRINT("Please Insert SD Card!\r\n")
        if( FSP_SUCCESS == R_SDHI_StatusGet(&g_sdmmc_ctrl, &Sts))
            card = Sts.card_inserted;
        else
        {
            APP_ERR_PRINT ("\r\nCan't check SD Slot\r\n")
        }
    }

    while (true)
    {
        if( (err = R_SDHI_StatusGet(&g_sdmmc_ctrl, &Sts)) == FSP_SUCCESS)
        {
            if(Sts.card_inserted != card)
            {
                if(Sts.card_inserted)
                {
                    if(!Sts.initialized)
                    {
                        R_SDHI_Close(&g_sdmmc_ctrl);
                        err = R_USB_Open(&g_basic_ctrl, &g_basic_cfg);
                        APP_PRINT("SD Inserted [MSD up (%d)]\r\n", err)
                    }
                    else
                        APP_PRINT("SD Inserted \r\n")
                }
                else
                {
                    if(Sts.initialized)
                    {
                        err = R_USB_Close(&g_basic_ctrl);
                        R_SDHI_Open(&g_sdmmc_ctrl, &g_sdmmc_cfg);
                        APP_PRINT("SD Ejected [MSD Down (%d)]\r\n", err)
                    }
                    else
                        APP_PRINT("SD Ejected\r\n")
                }

                card = Sts.card_inserted;
            }
        }
        else
        {
            APP_PRINT("SD Slot Check [%d] \r\n", err)
            err = R_SDHI_Open(&g_sdmmc_ctrl, &g_sdmmc_cfg);
            if (FSP_SUCCESS != err)
            {
                APP_ERR_PRINT("\r\nTry Reopening SDHI Failed [%d]\r\n", err)
            }
        }

        /* check for usb event */
        if( FSP_SUCCESS == R_USB_EventGet(&g_basic_ctrl, &event))
            switch(event)
            {
                case USB_STATUS_CONFIGURED :
                    APP_PRINT("USB MSD[%d] Configured Successfully\r\n", g_basic_cfg.module_number)
                break;

                case USB_STATUS_DETACH :
                //case USB_STATUS_SUSPEND :
                    APP_PRINT("USB MSD[%d] Removed Successfully\r\n", g_basic_cfg.module_number)
                break;

                default :
                break;
            }
    }
#if BSP_TZ_SECURE_BUILD
    /* Enter non-secure code */
    R_BSP_NonSecureEnter();
#endif
}

/*******************************************************************************************************************//**
 * This function is called at various points during the startup process.  This implementation uses the event that is
 * called right before main() to set up the pins.
 *
 * @param[in]  event    Where at in the start up process the code is currently at
 **********************************************************************************************************************/
void R_BSP_WarmStart(bsp_warm_start_event_t event)
{
    if (BSP_WARM_START_RESET == event)
    {
#if BSP_FEATURE_FLASH_LP_VERSION != 0

        /* Enable reading from data flash. */
        R_FACI_LP->DFLCTL = 1U;

        /* Would normally have to wait tDSTOP(6us) for data flash recovery. Placing the enable here, before clock and
         * C runtime initialization, should negate the need for a delay since the initialization will typically take more than 6us. */
#endif
    }

    if (BSP_WARM_START_POST_C == event)
    {
        /* C runtime environment and system clocks are setup. */

        /* Configure pins. */
        R_IOPORT_Open (&g_ioport_ctrl, g_ioport.p_cfg);
      #ifdef __USE_TTY__
        TTY_init();
      #endif
    }
}

#if BSP_TZ_SECURE_BUILD

BSP_CMSE_NONSECURE_ENTRY void template_nonsecure_callable ();

/* Trustzone Secure Projects require at least one nonsecure callable function in order to build (Remove this if it is not required to build). */
BSP_CMSE_NONSECURE_ENTRY void template_nonsecure_callable ()
{

}
#endif
