/* -------------------------------------------------------------------------+
 |                             Vekatech Ltd.                                |
 + -------------------------------------------------------------------------*/

/* --------------------------- COPYRIGHT INFORMATION -----------------------+
 |                                                                          |
 |                                                                          |
 |                                                                          |
 |                                                                          |
 + -------------------------------------------------------------------------*/

/* -------------------------------------------------------------------------+
 | Module        : <mandatory>
 | File name     : <mandatory>
 | Description   : <mandatory>
 | Reference(s)  : <mandatory>
 + -------------------------------------------------------------------------*/

#define  __TEMPLATE_C__ 0x0100

/* -------------------------------------------------------------------------+
 |   HEADER (INCLUDE) SECTION                                               |
 + -------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdarg.h>
#include "hal_data.h"
#include "SYSTEM_TTY/SYSTEM_TTY.h"

/* -------------------------------------------------------------------------+
 |   EXTERNAL DATA (+ meaning)                                              |
 + -------------------------------------------------------------------------*/

/* none*/

/* -------------------------------------------------------------------------+
 |   MACROS                                                                 |
 + -------------------------------------------------------------------------*/

#define SPACE       0
#define ITTEMS      1

/* -------------------------------------------------------------------------+
 |   TYPE DEFINITIONS                                                       |
 + -------------------------------------------------------------------------*/

/* none*/
 
/* -------------------------------------------------------------------------+
 |   GLOBAL CONSTANTS                                                       |
 + -------------------------------------------------------------------------*/

/* none*/

/* -------------------------------------------------------------------------+
 |   GLOBAL VARIABLES                                                       |
 + -------------------------------------------------------------------------*/

/* none*/

/* -------------------------------------------------------------------------+
 |   LOCAL FUNCTION PROTOTYPES                                              |
 + -------------------------------------------------------------------------*/

/* none*/

/* -------------------------------------------------------------------------+
 |   LOCAL  VARIABLES                                                       |
 + -------------------------------------------------------------------------*/
static uint8_t TXformat[TX_BUF_SIZE];
static uint8_t TXbuf[TX_BUF_SIZE];

static uint16_t TX_T;          //circular buffer tail (managed by the issuer i.e. the user)
static uint16_t volatile TX_H; //circular buffer head (managed by the consumer i.e. the UART)

static volatile uint16_t txIDX = 0;
static volatile uint8_t txSTAT = IDLE;

//---------------------------------

//static uint8_t RXbuf[RX_BUF_SIZE];

//static uint16_t volatile RX_T; //circular buffer tail (managed by the issuer i.e. the UART)
//static uint16_t RX_H;          //circular buffer head (managed by the consumer i.e. the user)

//static volatile uint16_t rxIDX = 0;
//static volatile uint8_t rxSTAT = IDLE;

/* -------------------------------------------------------------------------+
 |   LOCAL FUNCTIONS                                                        |                                         
 + -------------------------------------------------------------------------*/

static uint16_t buf(char op)
{
    register int txq_items;

    txq_items = TX_T - TX_H;

    if ( txq_items < 0 )
       txq_items += TX_BUF_SIZE;

    if ( txq_items < TX_BUF_SIZE-1 )
       return op? txq_items : (TX_BUF_SIZE - (txq_items + 1));
    else
       return 0;
}

/* -------------------------------------------------------------------------+
 |   GLOBAL FUNCTIONS                                                       |
 + -------------------------------------------------------------------------*/

fsp_err_t deinit_TTY(void)
{
    return R_SCI_UART_Close (&g_TTY_ctrl);
}

fsp_err_t TTY_init(void)
{
    TX_H = TX_T = 0;
    //RX_H = RX_T = 0;
    return R_SCI_UART_Open(&g_TTY_ctrl, &g_TTY_cfg);
}

/*****************************************************************************************************************
 *  @brief       print user \0 terminated message to terminal
 *  @param[in]   p_msg
 *  @retval      FSP_SUCCESS                Upon success
 *  @retval      Any Other Error code apart from FSP_SUCCESS,  Unsuccessful write operation
 ****************************************************************************************************************/
fsp_err_t TTY_print(void * p_msg)
{
    fsp_err_t err   = FSP_SUCCESS;

    if(p_msg)
    {
        FSP_CRITICAL_SECTION_DEFINE;

        FSP_CRITICAL_SECTION_ENTER;

        uint16_t len = (uint16_t)strlen(p_msg);
        uint16_t space = buf(SPACE);

        switch(txSTAT)
        {
            case XMIT:
            case XEND:
                if(len < space)
                {
                    if(len <= (TX_BUF_SIZE-TX_T))
                    {
                        memcpy(&TXbuf[TX_T], p_msg, len);
                        TX_T += len;
                        if(TX_T == TX_BUF_SIZE)
                            TX_T = 0;
                    }
                    else
                    { //split
                        memcpy(&TXbuf[TX_T], p_msg, TX_BUF_SIZE-TX_T);
                        TX_T = len-(TX_BUF_SIZE-TX_T);
                        memcpy(TXbuf, p_msg, TX_T);
                    }
                }
                else
                    err = FSP_ERR_INSUFFICIENT_SPACE;
            break;

            case IDLE:
            default:
                if(len < (TX_BUF_SIZE-1))
                {
                    memcpy(TXbuf, p_msg, len);
                    TX_T = len;
                    txIDX = TX_T;
                    txSTAT = XMIT;
                    err = R_SCI_UART_Write(&g_TTY_ctrl, TXbuf, len);

                    if(err != FSP_SUCCESS)
                    {
                        TX_T = 0;
                        txSTAT = IDLE;
                    }
                }
                else
                    err = FSP_ERR_INSUFFICIENT_SPACE;
            break;
        }
        FSP_CRITICAL_SECTION_EXIT;
    }
    else
        err = FSP_ERR_INVALID_ADDRESS;

    return err;
}

int TTY_printf(const char * sFormat, ...)
{
  int r;
  va_list ParamList;
  va_start(ParamList, sFormat);

  r = vsnprintf((char*)TXformat, sizeof(TXformat), sFormat, ParamList);
  if((r > 0) && r < sizeof(TXformat))
  {
      if(FSP_SUCCESS != TTY_print(TXformat))
          r = 0;
  }

  va_end(ParamList);
  return r;
}

void tty_ISR(uart_callback_args_t *p_args)
{
    //p_args->data

    switch(p_args->event)
    {
        case UART_EVENT_TX_COMPLETE:
            txIDX = buf(ITTEMS);

            if(txIDX)
            {
                if((TX_H + txIDX) >= TX_BUF_SIZE)
                    txIDX = TX_BUF_SIZE - TX_H;

                txSTAT = XMIT;
                R_SCI_UART_Write(&g_TTY_ctrl, &TXbuf[TX_H], txIDX);
            }
            else
            {
                TX_H = 0;
                txSTAT = IDLE;
            }
        break;

        case UART_EVENT_TX_DATA_EMPTY:
            TX_H += txIDX;
            txIDX = 0;

            if(TX_H >= TX_BUF_SIZE)
            {
                TX_H -= TX_BUF_SIZE;

                if(TX_H)
                {
                     //???
                }
            }

            txSTAT = (TX_T == TX_H)? XEND : XMIT;
        break;

        default:
        break;
    }
}

/* -------------------------------------------------------------------------+
 |   END                                                                    |
 + -------------------------------------------------------------------------*/

#undef __TEMPLATE_C__

